/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.download.jetty.server;

import java.util.logging.Logger;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.ow2.petals.microkernel.jbi.management.task.download.jetty.servlets.CorrectContentTypeServlet;
import org.ow2.petals.microkernel.jbi.management.task.download.jetty.servlets.CorrectContentTypeWithCharsetServlet;
import org.ow2.petals.microkernel.jbi.management.task.download.jetty.servlets.WithoutContentTypeServlet;
import org.ow2.petals.microkernel.jbi.management.task.download.jetty.servlets.WrongContentTypeServlet;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class JettyServer {

    /**
     * The logger
     */
    private Logger log = Logger.getLogger(JettyServer.class.getName());

    public static final int HEADER_BUFFER_SIZE = 4096 * 4;

    private static JettyServer instance = null;

    private CorrectContentTypeServlet correctContentTypeServlet;

    private CorrectContentTypeWithCharsetServlet correctContentTypeWithCharsetServlet;

    private WrongContentTypeServlet wrongContentTypeSevlet;
    
    private WithoutContentTypeServlet withoutContentTypeSevlet;

    private ServerConfig config;

    private Server server;

    /**
     * Creates a new instance of {@link JettyServer}
     * 
     * @param configContext
     * @param config
     * @param componentContext
     * @param logger
     * @throws AxisFault
     */
    private JettyServer(final ServerConfig config) {
        this.config = config;
        this.server = new Server();

        // jetty http connector configuration
        final ServerConnector nioConnector = new ServerConnector(this.server);
        nioConnector.setPort(this.config.getPort());

        // If we assign the host, we will only be able to contact server
        // on it. No value or a null one is a wildcard so connection is possible
        // on network interface
        // @see java.net.InetSocketAddress
        if (this.config.isRestrict()) {
            nioConnector.setHost(this.config.getHost());
        }
        this.server.addConnector(nioConnector);

    }

    public void createAllServlets() {
        // add welcome servlet
        final ContextHandlerCollection contexts = new ContextHandlerCollection();
        // create context handlers
        this.server.setHandler(contexts);

        createCorrectContentTypeServlet(contexts);

        createCorrectContentTypeWithCharsetServlet(contexts);

        createWrongContentTypeServlet(contexts);
        
        createWithoutContentTypeServlet(contexts);
    }

    private void createWithoutContentTypeServlet(final ContextHandlerCollection contexts) {
        final ServletContextHandler withoutContentTypeContext = new ServletContextHandler(contexts,
                "/WithoutContentType", ServletContextHandler.SESSIONS);
        this.withoutContentTypeSevlet = new WithoutContentTypeServlet();
        final ServletHolder repositoryServletHolder = new ServletHolder(withoutContentTypeSevlet);
        repositoryServletHolder.setName("WithoutContentTypeSevlet");
        repositoryServletHolder.setInitOrder(3);
        withoutContentTypeContext.addServlet(repositoryServletHolder, "/");
    }
    
    private void createWrongContentTypeServlet(final ContextHandlerCollection contexts) {
        final ServletContextHandler wrongContentTypeContext = new ServletContextHandler(contexts, "/WrongContentType",
                ServletContextHandler.SESSIONS);
        this.wrongContentTypeSevlet = new WrongContentTypeServlet();
        final ServletHolder repositoryServletHolder = new ServletHolder(wrongContentTypeSevlet);
        repositoryServletHolder.setName("WrongContentTypeSevlet");
        repositoryServletHolder.setInitOrder(1);
        wrongContentTypeContext.addServlet(repositoryServletHolder, "/");
    }
    
    private void createCorrectContentTypeServlet(final ContextHandlerCollection contexts) {
        final ServletContextHandler repositoryContext = new ServletContextHandler(contexts, "/CorrectContentType",
                ServletContextHandler.SESSIONS);
        this.correctContentTypeServlet = new CorrectContentTypeServlet();
        final ServletHolder correctContentTypeServletHolder = new ServletHolder(correctContentTypeServlet);
        correctContentTypeServletHolder.setName("CorrectContentTypeServlet");
        correctContentTypeServletHolder.setInitOrder(2);
        repositoryContext.addServlet(correctContentTypeServletHolder, "/");
    }

    private void createCorrectContentTypeWithCharsetServlet(final ContextHandlerCollection contexts) {
        final ServletContextHandler repositoryContext = new ServletContextHandler(contexts,
                "/CorrectContentTypeWithCharset", ServletContextHandler.SESSIONS);
        this.correctContentTypeWithCharsetServlet = new CorrectContentTypeWithCharsetServlet();
        final ServletHolder correctContentTypeWithCharsetServletHolder = new ServletHolder(
                correctContentTypeWithCharsetServlet);
        correctContentTypeWithCharsetServletHolder.setName("CorrectContentTypeWithCharsetServlet");
        correctContentTypeWithCharsetServletHolder.setInitOrder(2);
        repositoryContext.addServlet(correctContentTypeWithCharsetServletHolder, "/");
    }

    public static JettyServer getInstance() throws Exception {
        if (instance == null) {
            instance = new JettyServer(new ServerConfig());
            instance.createAllServlets();
            instance.start();
        }
        if (!instance.getServer().isStarted()) {
            instance.createAllServlets();
            instance.start();
        }

        return instance;
    }


    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.TransportListener#start()
     */
    public void start() throws Exception {
        this.log.info("Starting Jetty server...");
        this.log.info("Host : "
                + (!this.config.isRestrict() ? "*" : this.config.getHost() + " (restricted)")
                + " / Port : " + this.config.getPort());

        this.server.start();

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.TransportListener#stop()
     */
    public void stop() throws Exception {
        this.log.info("Stop and kill Jetty server...");

        this.server.stop();
        // this.server.destroy();

    }

    public Server getServer() {
        return server;
    }

    public ServerConfig getConfig() {
        return config;
    }

}
